/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package battle;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Arc2D;
import java.util.ArrayList;

/**
 *
 * @author jub
 * Interacts with World to give information about its sorroundings to SElement.
 */
public class Sensor {

    private boolean targetFound;
    private boolean targetDead;
    private boolean traceLost;
    private boolean targetLocked;
    private SElement sensedElement;
    private SElement element;
    private Arc2D.Float shape;
    private final World world;
    private float hsize = 10;
    private float wsize = 10;

    /**Constructor, creates sensor.
    @param _world World to perceive.
     * @param _element SElement to which it belongs.
     * 
     */
    Sensor(World _world, SElement _element) {
        element = _element;
        shape = new Arc2D.Float(
                element.getBounds(), element.getAngle(),
                90f, Arc2D.PIE);
        targetFound = false;
        targetLocked = false;
        targetDead = true;
        traceLost = true;
        world = _world;
        sensedElement = element;
    }

    /**Stores elements which intersects its area of effect.*/
    public void sense() {
        ArrayList<Element> el = world.getSensed(shape.getBounds2D());
        for (Element e : el) {
            if (e.getId() != element.getId() && e instanceof SElement) {
//                System.out.print(element.getId() + " TARGET DEAD = " + targetDead() + " TRACE LOST = " + traceLost() + "\n");
                targetFound = true;
                traceLost = false;
                targetDead = false;
                sensedElement = (SElement) e;
//                System.out.print(element.getId() + " sees " + sensedElement.getId() + "\n");
                return;
            } else {
                if (world.elementPresent(sensedElement)) {
                    traceLost = true;
                    targetDead = false;
                } else {
                    targetDead = true;
                    traceLost = false;
                }
                sensedElement = element;
                targetFound = false;
            }
        }
    }

    /**Returns angle in which the SElement should turn to be in line with target.*/
    public int getTargetAngle() {

        double dX = sensedElement.getX() - element.getX();
        double dY = sensedElement.getY() - element.getY();
        int ang = 0;
        Vector2D util = new Vector2D(dX, dY);
        double cos = element.getDir().dot(util)
                / (element.getDir().module() * util.module());
        if (dX > 0) {
            ang = (int) (Math.acos(cos) * 180 / Math.PI);
        } else {
            ang = -(int) (Math.acos(cos) * 180 / Math.PI);
        }
        if (Math.abs(ang) < 5) {
            targetLocked = true;
        }
        return ang;

    }

    /**Returns true if target is contained by area of effect.*/
    public boolean targetFound() {
        if (sensedElement != element && sensedElement != null) {
            return true;
        } else {
            return false;
        }
    }

    /**Returns true if target is aligned with SElement.*/
    public boolean targetLocked() {
        return targetLocked;
    }

    /**Returns true if target is no longer an element of the world.*/
    public boolean targetDead() {
        return targetDead;
    }

    /**Returns true if target is no longer in area of effect.*/
    public boolean traceLost() {
        return traceLost;
    }

    /**Defines how to draw this element.
    @param g Graphics2D element.
     */
    public void draw(Graphics2D g) {
        shape.setAngleStart(360f - element.getAngle() - 45);
        shape.setFrame((float) element.getX() - (float) element.getSize() * wsize / 2,
                (float) element.getY() - (float) element.getSize() * hsize / 2,
                (float) element.getSize() * wsize,
                (float) element.getSize() * hsize);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        g.setPaint(new Color(
                element.getColor().getRed() / 255f,
                element.getColor().getGreen() / 255f,
                element.getColor().getBlue() / 255f,
                0.3f));
        g.fill(shape);
        g.setColor(Color.LIGHT_GRAY);
    }
}
