package glgame;

import org.lwjgl.input.Keyboard;
import org.lwjgl.opengl.GL11;

import glmodel.GL_Vector;

public class Nave extends GraphicsEntitie{
	
	protected GL_Vector directionX;
	protected GL_Vector directionY;
	protected GL_Vector directionZ;
	
	protected double angleXZ;
	protected double angleYZ;
	protected double angleXY;
	
	
	/**
	 * 
	 * 
	 * @param fileName Identificador del modelo 3D
	 */
	public Nave(String fileName){
		super(fileName);
		location=new GL_Vector();
		directionX=new GL_Vector(1,0,0);
		directionY=new GL_Vector(0,1,0);
		directionZ=new GL_Vector(0,0,1);
		
		
	}
	/**
	 * 
	 * 
	 * @param fileName identificador del modelo3D
	 * @param xpos posicion inicial x
	 * @param ypos posicion inicial y
	 * @param zpos posicion inicial z
	 */
	public Nave (String fileName,float xpos, float ypos, float zpos){
		super(fileName);
		location=new GL_Vector(xpos, ypos, zpos);
		directionX=new GL_Vector(1,0,0);
		directionY=new GL_Vector(0,1,0);
		directionZ=new GL_Vector(0,0,1);
		
	}

	/**
	 * Traslada el objeto a la direcion x,y,z
	 * 
	 * @param x Nueva posicion x
	 * @param y Nueva posicion y
	 * @param z Nueva posicion z
	 */
	public void setLocation(float x, float y, float z){
		location=new GL_Vector(x,y,z);
	}
	
	/**
	 * Traslada el objeto a la direcion vector(x,y,z)
	 * @param v nuevo vector posicion
	 */
	public void setLocation(GL_Vector v){
		location=new GL_Vector(v);
	}
	/**
	 * Sumar posicion x,y,z
	 * @param x 
	 * @param y
	 * @param z
	 */
	public void plusLocation(float x, float y, float z){
		location.add(new GL_Vector(x,y,z));
		
	}
	/**
	 * Sumar posicion con vector(x,y,z)
	 * @param v
	 */
	public void plusLocation(GL_Vector v){
		location.add(v);
		
	}
	
	/**
	 * Devuelve la posicion en forma de arreglo
	 * @return devuelve arreglo de posicion
	 */
	public float[] getLocation(){
		return (new float[]{location.x, location.y, location.z});
	}
	/**
	 * Devuelve la posicion en forma de vector
	 * @return devuelve vector posicion
	 */
	public GL_Vector getLocationVector(){
		return location;
	}
	/**
	 * 
	 * 
	 * @return parametros de vector arriba
	 */
	public float[] getUp(){
		return(new float[]{directionY.x,directionY.y,directionY.z});
	}
	/**
	 * 
	 * @return vector arriba
	 */
	public GL_Vector getUpVector(){
		return directionY;
	}
	/**
	 * 
	 * @return parametros del vector frontal
	 */
	public float[] getFront(){
		return(new float[]{directionZ.x,directionZ.y,directionZ.z});
	}
	/**
	 * 
	 * @return Vector frontal
	 */
	public GL_Vector getFrontVector(){
		return directionZ;
	}

	/**
	 * 
	 * @return parametros del vector lado
	 */
	public float[] getSide(){
		return(new float[]{directionX.x,directionX.y,directionX.z});
	}
	/**
	 * 
	 * @return Vector lado
	 */
	public GL_Vector getSideVector(){
		return directionX;
	}
	
	/**
	 * Mover en direccion X global
	 * @param distance distancia x
	 */
	public void moveX(float distance){
		
		location.add(new GL_Vector(distance,0,0));
		
	}
	/**
	 * Mover en direccion Y global
	 * @param distance distancia y
	 */
	public void moveY(float distance){
		
		location.add(new GL_Vector(0,distance,0));
		
	}
	/**
	 * Mover en direccion z global
	 * @param distance distancia z
	 */
	public void moveZ(float distance){
	
		location.add(new GL_Vector(0,0,distance));
	
	}
	/**
	 * Moverse hacia el frente
	 * @param distance distancia que el objeto de mueve hacia el frente
	 */
	public void moveFront(float distance){
		location.add(GL_Vector.multiply(directionZ, distance));
	}
	/**
	 * Moverse hacia atras
	 * @param distance distancia que el objeto de mueve hacia atras
	 */
	public void moveBack(float distance){
		location.add(GL_Vector.multiply(directionZ, -distance));
		
	}
	/**
	 * Rotacion sobre SU plano xz
	 * @param angle rotacion de su plano XZ
	 */
	public void rotateXZ(float angle){

		directionX=GL_Vector.RotationVector3D((angle*90*(float)Math.PI)/180, directionX, directionY);
		directionZ=GL_Vector.RotationVector3D((angle*90*(float)Math.PI)/180, directionZ, directionY);
	}
	/**
	 * Rotacion sobre SU plano xy
	 * @param angle rotacion de su plano XY
	 */	
	public void rotateXY(float angle){

		directionX=GL_Vector.RotationVector3D((angle*90*(float)Math.PI)/180, directionX, directionZ);
		directionY=GL_Vector.RotationVector3D((angle*90*(float)Math.PI)/180, directionY, directionZ);
	}
	/**
	 * Rotacion sobre SU plano yz
	 * @param angle rotacion de su plano YZ
	 */
	public void rotateYZ(float angle){

		directionY=GL_Vector.RotationVector3D((angle*90*(float)Math.PI)/180, directionY, directionX);
		directionZ=GL_Vector.RotationVector3D((angle*90*(float)Math.PI)/180, directionZ, directionX);
	}
	
	/**
	 * roll (rotation around z) : atan2(xy, xx)
		pitch (rotation around y) : -arcsin(xz)
		yaw (rotation around x) : atan2(yz,zz)
		
			xx, yx, zx, px;
			xy, yy, zy, py;
			xz, yz, zz, pz;
			0, 0, 0, 1
			
			
	 * 
	 */
	public void getAngle(){
		
		if (directionZ.x!=1||directionZ.x!=-1){
			angleXZ=-Math.asin(directionZ.x)*180/Math.PI;
			angleYZ=(Math.atan2(directionZ.y/Math.cos(angleXZ*Math.PI/180), directionZ.z/Math.cos(angleXZ*Math.PI/180))*180/Math.PI);
			angleXY=(Math.atan2(directionY.x/Math.cos(angleXZ*Math.PI/180), directionX.x/Math.cos(angleXZ*Math.PI/180))*180/Math.PI);
			
		}
		else{
			angleXY=0;
			if(directionZ.x==1){
				angleXZ=90;
				angleYZ=(Math.atan2(directionX.y, directionX.z)*180/Math.PI);
			}
			else{
				angleXZ=-90;
				angleYZ=-(Math.atan2(directionX.y, directionX.z)*180/Math.PI);
			}
		}
	}
	
	/**
	 * render del modelo3D
	 */
	public void render(){
		GL11.glPushMatrix();
        {
        	GL11.glTranslatef(location.x,location.y,location.z);
        	
        	getAngle();
        	

        	
        	GL11.glRotatef((float)-angleYZ, 1, 0, 0);
        	GL11.glRotatef((float)-angleXZ, 0, 1, 0);
        	GL11.glRotatef((float)-angleXY, 0, 0, 1);
        	GL11.glScalef(.1f,.1f, .1f);
        	
        	models3Dcompilation.getModels3Dcompilation().render(mainModel);
        	
        }
        GL11.glPopMatrix();
			
	}
	
	
	 /**
	  * Adjust the Camera position based on keyboard arrow key input.
	  * These are repeating events (camera will move as long as key is held
	  * down).  Moves the camera according to time elapsed, so camera motion
	  * is consistent across varied cpu speeds.
	  * 
	  * @param seconds  seconds elapsed since last frame was drawn
	  */
	 public void handleNavKeys(float seconds)
	 {
		 //boolean altKeyDown = Keyboard.isKeyDown(Keyboard.KEY_RMENU)||Keyboard.isKeyDown(Keyboard.KEY_LMENU);
		 //boolean ctrlKeyDown = Keyboard.isKeyDown(Keyboard.KEY_RCONTROL)||Keyboard.isKeyDown(Keyboard.KEY_LCONTROL);
		 
		 // Basic motion
		 // Turn left
		 if (Keyboard.isKeyDown(Keyboard.KEY_LEFT)) {
			 //camera.RotateV(camSpeedR*seconds);
			 //moveX(seconds);
			 rotateXZ(seconds);
		 }
		 // Turn right
		 if (Keyboard.isKeyDown(Keyboard.KEY_RIGHT)) {
			// camera.RotateV(-camSpeedR*seconds);
			 //moveX(-seconds);
			 rotateXZ(-seconds);
		 }
		 // move forward
		 if (Keyboard.isKeyDown(Keyboard.KEY_UP)) {
			 //camera.MoveForward(-camSpeedXZ*seconds);
			 moveFront(seconds);
		 }
		 // move backward
		 if (Keyboard.isKeyDown(Keyboard.KEY_DOWN)) {
			 //camera.MoveForward(camSpeedXZ*seconds);
			 moveFront(-seconds);
		 }
		 // move down
		 if (Keyboard.isKeyDown(Keyboard.KEY_PRIOR)) {
			 //camera.MoveUpward(camSpeedY*seconds);
		 }
		 // move up
		 if (Keyboard.isKeyDown(Keyboard.KEY_NEXT)) {
			// camera.MoveUpward(-camSpeedY*seconds);
		 }
		 
		 // additional motion
		 // Pan left
		 if (Keyboard.isKeyDown(Keyboard.KEY_A)) {
			 //camera.StrafeRight(-camSpeedXZ*seconds);
			 rotateXY(-seconds);
		 }
		 // Pan right
		 if (Keyboard.isKeyDown(Keyboard.KEY_D)) {
			 //camera.StrafeRight(camSpeedXZ*seconds);
			 rotateXY(seconds);
		 }
		 // tilt down
		 if (Keyboard.isKeyDown(Keyboard.KEY_S)) {
			// camera.RotateX(-camSpeedR*seconds);
			 rotateYZ(seconds);
		 }
		 // tilt up
		 if (Keyboard.isKeyDown(Keyboard.KEY_W)) {
			 //camera.RotateX(camSpeedR*seconds);
			 rotateYZ(-seconds);
		 }
		 // tilt left
		 if (Keyboard.isKeyDown(Keyboard.KEY_Z)) {
			 //camera.RotateZ(camSpeedR*seconds);
		 }
		 // tilt right
		 if (Keyboard.isKeyDown(Keyboard.KEY_C)) {
			// camera.RotateZ(-camSpeedR*seconds);
		 }
	 }
	
	
}
